#include "StdAfx.h"
#include "FTUtil.h"

/* Defines */
#define SHPHONELIB_DLL_NAME		_T("shphonelib.dll")
#define FT_GETFILELIST_NAME		_T("?FT_GetFileList@CshphoneClientlib@@SAHPAUWSIMFileList_TAG@@@Z")
#define FT_PROCESS_BEGIN_NAME	_T("?FT_ProcessBegin@CshphoneClientlib@@SAHXZ")
#define FT_PROCESS_END_NAME		_T("?FT_ProcessEnd@CshphoneClientlib@@SAXXZ")
#define FT_RECVFILE_NAME		_T("?FT_RecvFile@CshphoneClientlib@@SAHPB_W0K@Z")
#define FT_SENDFILE_NAME		_T("?FT_SendFile@CshphoneClientlib@@SAHPB_W0K0@Z")

/* Typedefs */
typedef int (__cdecl * PFT_GETFILELIST)(struct WSIMFileList_TAG *);
typedef int (__cdecl * PFT_PROCESSBEGIN)(void);
typedef void (__cdecl * PFT_PROCESSEND)(void);
typedef int (__cdecl * PFT_RECVFILE)(wchar_t const *,wchar_t const *, unsigned long);
typedef int (__cdecl * PFT_SENDFILE)(wchar_t const *,wchar_t const *, unsigned long, wchar_t const *);

typedef struct _FT_FUNC_ENTRY
{
	PFT_PROCESSBEGIN	pfnProcessBegin;
	PFT_PROCESSEND		pfnProcessEnd;
	PFT_GETFILELIST		pfnGetFileList;
	PFT_RECVFILE		pfnRecvFile;
	PFT_SENDFILE		pfnSendFile;
} FT_FUNC_ENTRY;

/* Global Variables */
static	BOOL	g_bInitialized	= FALSE;
static	HMODULE	g_hModule		= NULL;
static	FT_FUNC_ENTRY g_Entry;

////////////////////////////////////////////////////////////

BOOL
FTUtil_Initialize()
{
	BOOL	bRet = FALSE;

	if (!g_bInitialized) {

		HMODULE	hModule;

		hModule = LoadLibrary(SHPHONELIB_DLL_NAME);

		if (hModule != NULL) {
			PFT_GETFILELIST		pfnGetFileList = NULL;
			PFT_PROCESSBEGIN	pfnProcessBegin = NULL;
			PFT_PROCESSEND		pfnProcessEnd = NULL;
			PFT_RECVFILE		pfnRecvFile = NULL;
			PFT_SENDFILE		pfnSendFile = NULL;

			pfnProcessBegin = (PFT_PROCESSBEGIN) GetProcAddress(hModule, FT_PROCESS_BEGIN_NAME);
			pfnProcessEnd = (PFT_PROCESSEND)GetProcAddress(hModule, FT_PROCESS_END_NAME);
			pfnGetFileList = (PFT_GETFILELIST)GetProcAddress(hModule, FT_GETFILELIST_NAME);
			pfnRecvFile = (PFT_RECVFILE)GetProcAddress(hModule, FT_RECVFILE_NAME);
			pfnSendFile = (PFT_SENDFILE)GetProcAddress(hModule, FT_SENDFILE_NAME);

			
			if (pfnProcessBegin != NULL	&& 
				pfnProcessEnd != NULL	&&
				pfnGetFileList != NULL	&&
				pfnRecvFile != NULL		&&
				pfnSendFile != NULL ) {

				g_Entry.pfnProcessBegin = pfnProcessBegin;
				g_Entry.pfnProcessEnd	= pfnProcessEnd;
				g_Entry.pfnGetFileList	= pfnGetFileList;
				g_Entry.pfnRecvFile		= pfnRecvFile;
				g_Entry.pfnSendFile		= pfnSendFile;

				g_hModule = hModule;
				g_bInitialized = TRUE;

				bRet = TRUE;
			}
			else {
				FreeLibrary(hModule);
				hModule = NULL;
			}
		}
	}

	return bRet;
}

BOOL
FTUtil_Terminate()
{
	BOOL	bRet = FALSE;

	if (g_bInitialized) {
		if (g_hModule != NULL) {
			FreeLibrary(g_hModule);
			g_hModule = NULL;
		}

		g_bInitialized = FALSE;
		bRet = TRUE;
	}

	return bRet;
}

int
FTUtil_ProcessBegin()
{
	int	iRet = 0;

	if (g_bInitialized) {
		iRet = (* g_Entry.pfnProcessBegin)();
	}
	return iRet;
}

void
FTUtil_ProcessEnd()
{
	if (g_bInitialized) {
		 (* g_Entry.pfnProcessEnd)();
	}
}

int
FTUtil_GetFileList(
	struct WSIMFileList_TAG* pTagList
)
{
	int	iRet = 0;

	if (g_bInitialized) {
		iRet = (* g_Entry.pfnGetFileList)(pTagList);
	}
	return iRet;
}

int
FTUtil_RecvFile(
	LPWSTR			pDestFile,
	LPWSTR			pSrcFile,
	unsigned long	ulFileSize
)
{
	int	iRet = 0;

	if (g_bInitialized) {
		iRet = (* g_Entry.pfnRecvFile)(pDestFile, pSrcFile, ulFileSize);
	}
	return iRet;
}

int
FTUtil_SendFile(
	LPWSTR	pDestFile,
	LPWSTR	pSrcFile,
	unsigned long ulFileSize,
	LPWSTR	pLocalTimeStr		// "%04d%02d%02d%02d%02d%02d"`, YYYYMMDDhhmmss
)
{
	int	iRet = 0;

	if (g_bInitialized) {
		iRet = (* g_Entry.pfnSendFile)(pDestFile, pSrcFile, ulFileSize, pLocalTimeStr);
	}
	return iRet;
}
